local name = ...
SnorkHudDB = {}

local _sml = LibStub:GetLibrary("LibSharedMedia-3.0")

local _uHealth = UnitHealth
local _uHealthMax = UnitHealthMax
local _uPower = UnitPower
local _uPowerMax = UnitPowerMax

local _frame = CreateFrame("Frame", "SnorkHudFrame", UIParent)
_frame.ready = false
function _frame:init()
	--self:loadVariables()

	self:CreateFrame("SnorkHUDHealthFrame", self.db.healthFrame)
	self:CreateFrame("SnorkHUDPowerFrame", self.db.powerFrame)
	self:initOptionsPanel()
	self.ready = true
	self:initFrames()
	self:registerEvents()
end

function _frame:initFrames()
--healthFrame
	if self.db.healthFrame.enabled then
		self:initHealth()
		SnorkHUDHealthFrame:Show()
	else
		SnorkHUDHealthFrame:Hide()
	end
--powerFrame
	if self.db.powerFrame.enabled then
		self:initPower()
		SnorkHUDPowerFrame:Show()
	else
		SnorkHUDPowerFrame:Hide()
	end
end

function _frame:initHealth()
	SnorkHUDHealthFrame.text:SetFont(_sml:Fetch("font", self.db.healthFrame.font) or "Fonts\\MORPHEUS.ttf", self.db.healthFrame.size, self.db.healthFrame.flags)
	SnorkHUDHealthFrame.text:SetVertexColor(self.db.healthFrame.r, self.db.healthFrame.g, self.db.healthFrame.b)
	self:updateHealth()
	SnorkHUDHealthFrame:SetWidth(SnorkHUDHealthFrame.text:GetStringWidth())
	SnorkHUDHealthFrame:SetHeight(SnorkHUDHealthFrame.text:GetStringHeight())
	SnorkHUDHealthFrame:SetPoint("TOPLEFT", self.db.healthFrame.x, -self.db.healthFrame.y)
	SnorkHUDHealthFrame:SetAlpha(self.db.healthFrame.a)
end

function _frame:initPower()
	SnorkHUDPowerFrame.text:SetFont(_sml:Fetch("font", self.db.powerFrame.font) or "Fonts\\MORPHEUS.ttf", self.db.powerFrame.size, self.db.powerFrame.flags)
	if self.db.powerFrame.overridecolor then
		SnorkHUDPowerFrame.text:SetVertexColor(self.db.powerFrame.r, self.db.powerFrame.g, self.db.powerFrame.b)
	else
		self:setPowerType()
	end
	self:updatePower()
	SnorkHUDPowerFrame:SetWidth(SnorkHUDPowerFrame.text:GetStringWidth())
	SnorkHUDPowerFrame:SetHeight(SnorkHUDPowerFrame.text:GetStringHeight())
	SnorkHUDPowerFrame:SetPoint("TOPLEFT", self.db.powerFrame.x, -self.db.powerFrame.y)
	SnorkHUDPowerFrame:SetAlpha(self.db.powerFrame.a)
end

function _frame:setPowerType()
	if self.db.powerFrame.overridecolor or not self.ready then
		return
	end

	local type = UnitPowerType("player")
	local color = PowerBarColor[type]
	SnorkHUDPowerFrame.text:SetVertexColor(color.r, color.g, color.b)
end

function _frame:registerEvents()
	self:SetScript("OnUpdate", self.OnUpdate)
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("UPDATE_SHAPESHIFT_FORM")
	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
end

function _frame:OnEvent(e, arg1)
	if (e == "ADDON_LOADED" and name == arg1) then
		self:loadVariables()
		_frame:UnregisterEvent(e)
	elseif (e == "PLAYER_LOGIN") then
		self:init()
		_frame:UnregisterEvent(e)
	elseif (e == "UPDATE_SHAPESHIFT_FORM" or e == "PLAYER_SPECIALIZATION_CHANGED") then
		self:setPowerType()
	elseif (e == "PLAYER_REGEN_DISABLED") then
		if (self.db.healthFrame.enabled and not self.db.healthFrame.ooc) then
			SnorkHUDHealthFrame:Show()
			self:updateHealth()
		end

		if (self.db.powerFrame.enabled and not self.db.powerFrame.ooc) then
			SnorkHUDPowerFrame:Show()
			self:updatePower()
		end
	elseif (e == "PLAYER_REGEN_ENABLED") then
		if (self.db.healthFrame.enabled and not self.db.healthFrame.ooc) then
			SnorkHUDHealthFrame:Hide()
		end

		if (self.db.powerFrame.enabled and not self.db.powerFrame.ooc) then
			SnorkHUDPowerFrame:Hide()
		end
	end
end


local e = 0
local updateRate = 0.15
--local updateRateOOC = 1.0
function _frame:OnUpdate(elapsed)
	e = e + elapsed
	if (e > updateRate) then
		e = 0
		self:updateHealth()
		self:updatePower()
	end
end

function _frame:toPct(current, max)
	if(max == 0) then
		max = 1
	end
	return floor((current / max) * 100)
end

function _frame:toThousands(value)
	if(value > 10000) then
		return string.format("%.1fk", value / 1000)
	else
		return value
	end
end

function _frame:updateHealth()
	if self.db.healthFrame.pct then
		SnorkHUDHealthFrame.text:SetFormattedText("%d%%", self:toPct(_uHealth("player"), _uHealthMax("player")))
	else
		SnorkHUDHealthFrame.text:SetText(self:toThousands(_uHealth("player")))
	end
end

function _frame:updatePower()
	if self.db.powerFrame.pct then
		SnorkHUDPowerFrame.text:SetFormattedText("%d%%", self:toPct(_uPower("player"), _uPowerMax("player")))
	else
		SnorkHUDPowerFrame.text:SetText(self:toThousands(_uPower("player")))
	end
end

function _frame:CreateFrame(name, values)
	local frame = CreateFrame("Frame", name, UIParent)
	frame.values = values

	local width = 200
	local height = 50

	frame:SetWidth(width)
	frame:SetHeight(height)
	frame:SetClampedToScreen(true)
	frame:SetMovable(true)
	frame:SetFrameStrata("LOW")
	frame:SetPoint("CENTER", values.x, -values.y)
	frame.text = frame:CreateFontString()
	frame.text:SetWidth(width)
	frame.text:SetHeight(height)
	frame.text:SetPoint("CENTER", frame, "CENTER")
end

function _frame:loadVariables()
	self.playerName = format("%s - %s", UnitName("player"), GetRealmName())
	--defaults--
	if not SnorkHudDB[self.playerName] then
		self:doDefaults(self.playerName)
	end

	self.db = SnorkHudDB[self.playerName]
end

function _frame:doDefaults(playerName)
	SnorkHudDB[playerName] = {}
	SnorkHudDB[playerName]["healthFrame"] = {}
	SnorkHudDB[playerName]["healthFrame"].enabled = true
	SnorkHudDB[playerName]["healthFrame"].font = "Prototype"
	SnorkHudDB[playerName]["healthFrame"].size = 18
	SnorkHudDB[playerName]["healthFrame"].flags = "OUTLINE"
	SnorkHudDB[playerName]["healthFrame"].r = 0
	SnorkHudDB[playerName]["healthFrame"].g = 1
	SnorkHudDB[playerName]["healthFrame"].b = 0
	SnorkHudDB[playerName]["healthFrame"].a = 1
	SnorkHudDB[playerName]["healthFrame"].x = 900
	SnorkHudDB[playerName]["healthFrame"].y = 750
	SnorkHudDB[playerName]["healthFrame"].ooc = true
	SnorkHudDB[playerName]["healthFrame"].pct = false

	SnorkHudDB[playerName]["powerFrame"] = {}
	SnorkHudDB[playerName]["powerFrame"].enabled = true
	SnorkHudDB[playerName]["powerFrame"].font = "Prototype"
	SnorkHudDB[playerName]["powerFrame"].size = 18
	SnorkHudDB[playerName]["powerFrame"].flags = "OUTLINE"
	SnorkHudDB[playerName]["powerFrame"].overridecolor = false
	SnorkHudDB[playerName]["powerFrame"].r = 0
	SnorkHudDB[playerName]["powerFrame"].g = 0
	SnorkHudDB[playerName]["powerFrame"].b = 1
	SnorkHudDB[playerName]["powerFrame"].a = 1
	SnorkHudDB[playerName]["powerFrame"].x = 1100
	SnorkHudDB[playerName]["powerFrame"].y = 750
	SnorkHudDB[playerName]["powerFrame"].ooc = true
	SnorkHudDB[playerName]["powerFrame"].pct = false
end

--OPTIONS PANEL--
local _optionsPanel = CreateFrame("Frame", nil, InterfaceOptionsFramePanelContainer)
function _frame:createCheckBox(parent, label, desc, state, lambda)
	local checkbox = CreateFrame("CheckButton", "SnorkHudCBox" .. label, parent, "InterfaceOptionsCheckButtonTemplate")
	checkbox:SetChecked(state or false)
	checkbox.tooltipText = desc
	checkbox.SetValue = lambda
	checkbox.title = string.sub(label, 2)
	checkbox:SetScript("OnClick",
	function(self, event, arg1)
		self.SetValue(self:GetChecked())
	end)

	_G[checkbox:GetName() .. "Text"]:SetText(format("%s", checkbox.title))
	return checkbox
end

function _frame:createSlider(parent, label, desc, state, lambda, min, max, step)
	local slider = CreateFrame("Slider", "SnorkHudSlider" .. label, parent, "OptionsSliderTemplate")
	slider.tooltipText = desc
	slider.callback = lambda
	slider:SetObeyStepOnDrag(true)
	slider:SetValueStep(step)
	slider:SetMinMaxValues(min, max)
	slider.minText = _G[slider:GetName() .. "Low"]
	slider.maxText = _G[slider:GetName() .. "High"]
	slider.minText:SetText(min)
	slider.maxText:SetText(max)
	slider:SetValue(state)
	slider.title = string.sub(label, 2)
	slider:SetScript("OnValueChanged", function(self, value)
		self.callback(value)
		_G[slider:GetName() .. "Text"]:SetText(format("%s (%d)", self.title, self:GetValue()))
	end)

	slider:SetScript("OnMouseWheel", function(self, value)
		local newvalue = self:GetValue() + (value * self:GetValueStep())
		--self.callback(value)
		self:SetValue(newvalue)
		_G[slider:GetName() .. "Text"]:SetText(format("%s (%d)", self.title, self:GetValue()))
	end)

	_G[slider:GetName() .. "Text"]:SetText(format("%s (%d)", slider.title, slider:GetValue()))

	return slider
end

local function showColorPicker(r, g, b, a, callback)
  ColorPickerFrame:SetColorRGB(r,g,b)
  ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = (a ~= nil), a
  ColorPickerFrame.previousValues = {r,g,b,a}
  ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = callback, callback, callback
  ColorPickerFrame:Hide() -- Need to run the OnShow handler.
  ColorPickerFrame:Show()
end

function _frame:createColorPicker(parent, label, desc, state, lambda)
	local colorpicker = CreateFrame("Button", "SnorkHudButton" .. label, parent, "UIPanelButtonTemplate")
	colorpicker:SetSize(96, 32)
	colorpicker.color = state
	colorpicker.recolorTexture = lambda
	colorpicker:SetText(label)

	colorpicker:EnableMouse(true)
	colorpicker:SetScript("OnMouseDown", function(self, button,...)
		if(button == "LeftButton") then
			showColorPicker(self.color.r, self.color.g, self.color.b, self.color.a, self.recolorTexture)
		end
	end)

	return colorpicker
end

function _frame:createDropDown(parent, label, desc, state, items, lambda)
local dropdown =  CreateFrame("Frame", label, parent, "UIDropDownMenuTemplate")
	_G[dropdown:GetName() .. "Text"]:SetText(label)
	dropdown.info = {}
	UIDropDownMenu_SetText(dropdown, state)
	dropdown.items = items

	UIDropDownMenu_Initialize(dropdown,
	function(self, level, items)
		local info = UIDropDownMenu_CreateInfo()

		for k, v in pairs(self.items) do
			info.text = v
			info.notCheckable = true
			info.arg1 = v
			info.func = lambda
			UIDropDownMenu_AddButton(info)
		end
	end)

	return dropdown
end

function _frame:initOptionsPanel()
	_optionsPanel.name = "Snork HUD"
	InterfaceOptions_AddCategory(_optionsPanel)
	--title--
	local hFrameOptions=CreateFrame("Frame", "SnorkHOptionsTitle", _optionsPanel)
		hFrameOptions:SetPoint("TOPLEFT", 10, -55)
		hFrameOptions:SetWidth(150)
		hFrameOptions:SetHeight(50)

	local addonOptionsTitle = hFrameOptions:CreateFontString(nil, "OVERLAY", "GameFontNormal")
			addonOptionsTitle:SetText("Snork HUD")
			addonOptionsTitle:SetPoint("TOPLEFT", 0, 40)
			addonOptionsTitle:SetFont("Fonts\\FRIZQT__.TTF", 24)
	--health title--
	local hFrameOptionsFS = hFrameOptions:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		hFrameOptionsFS:SetText("Health Frame")
		hFrameOptionsFS:SetPoint("TOPLEFT", 0, 0)
		hFrameOptionsFS:SetFont("Fonts\\FRIZQT__.TTF", 16)
	--health enable--
	local checkboxHEnable = self:createCheckBox(hFrameOptions, "HEnable", "Show health value", self.db.healthFrame.enabled,
	function(val)
		self.db.healthFrame.enabled = val
		self:initFrames()
	end)
	checkboxHEnable:SetScale(1.5)
	checkboxHEnable:SetPoint("TOPLEFT", 4, -12)
	--health ooc--
	local checkboxHOOC = self:createCheckBox(hFrameOptions, "HShow out of combat", "Show health value when out of combat", self.db.healthFrame.ooc,
	function(val)
		self.db.healthFrame.ooc = val
		if not val and not UnitAffectingCombat("player") then
			SnorkHUDHealthFrame:Hide()
		else
			SnorkHUDHealthFrame:Show()
		end
	end)
	checkboxHOOC:SetPoint("TOPLEFT", 16, -56)
	--health pct--
	local checkboxHPercent = self:createCheckBox(hFrameOptions, "HPercentage", "Show health value as percentage", self.db.healthFrame.pct,
	function(val)
		self.db.healthFrame.pct = val
	end)
	checkboxHPercent:SetPoint("TOPLEFT", 16, -76)
	--health fontsize--
	local sliderHFontSize = self:createSlider(hFrameOptions, "HFont size", "Text size for health value", self.db.healthFrame.size,
	function(val)
		self.db.healthFrame.size = val
		self:initHealth()
	end, 8, 60, 2)
	sliderHFontSize:SetPoint("TOPLEFT", 16, -120)
	--health x position--
	local editHX = self:createSlider(hFrameOptions, "HX pos", "Horizontal position for health value", self.db.healthFrame.x,
	function(val)
		self.db.healthFrame.x = val
		self:initHealth()
	end, 0, floor(GetScreenWidth()), 2)
	editHX:SetPoint("TOPLEFT", 16, -152)
	--health y position--
	local editHX = self:createSlider(hFrameOptions, "HY pos", "Vertical position for health value", self.db.healthFrame.y,
	function(val)
		self.db.healthFrame.y = val
		self:initHealth()
	end, 0, floor(GetScreenHeight()), 2)
	editHX:SetPoint("TOPLEFT", 16, -184)
	--health color picker--
	local colorH = self:createColorPicker(hFrameOptions, "Health color", "Color for health value", self.db.healthFrame,
	function(restore)
		local newR, newG, newB, newA;
		if restore then
			newR, newG, newB, newA = unpack(restore);
		else
			newA, newR, newG, newB = OpacitySliderFrame:GetValue(), ColorPickerFrame:GetColorRGB();
		end

		self.db.healthFrame.r = newR
		self.db.healthFrame.g = newG
		self.db.healthFrame.b = newB
		self.db.healthFrame.a = newA
		self:initHealth()
	end)
	colorH:SetPoint("TOPLEFT", 32, -208)
	--health outline
	local healthOutline = {"MONOCHROME", "OUTLINE", "THICKOUTLINE"}
	local healthDropOutline = self:createDropDown(hFrameOptions, "HealthOutlineDrop", "Outline of health value", self.db.healthFrame.flags, healthOutline,
	function(key)
		self.db.healthFrame.flags = key.value
		self:initHealth()

		UIDropDownMenu_SetSelectedName(HealthOutlineDrop, key.text, key.value)
		UIDropDownMenu_SetText(HealthOutlineDrop, key.value)
		CloseDropDownMenus()
	end)
	healthDropOutline:SetPoint("TOPLEFT", 0, -240)
	--health font--
	local healthfonts = _sml:List("font")
	local healthFontOption = self:createDropDown(hFrameOptions, "HealthFontsDrop", "Outline of health value", self.db.healthFrame.font, healthfonts,
	function(key)
		self.db.healthFrame.font = key.value
		self:initHealth()

		UIDropDownMenu_SetSelectedName(HealthFontsDrop, key.text, key.value)
		UIDropDownMenu_SetText(HealthFontsDrop, key.value)
		CloseDropDownMenus()
	end)
	healthFontOption:SetPoint("TOPLEFT", 0, -280)

	--POWAH--
	--power title--
	local pFrameOptionsFS = hFrameOptions:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		pFrameOptionsFS:SetText("Power Frame")
		pFrameOptionsFS:SetPoint("TOPLEFT", 400, 0)
		pFrameOptionsFS:SetFont("Fonts\\FRIZQT__.TTF", 16)
	--power enable--
	local checkboxPEnable = self:createCheckBox(hFrameOptions, "PEnable", "Show power value", self.db.powerFrame.enabled,
	function(val)
		self.db.powerFrame.enabled = val
		self:initFrames()
	end)
	checkboxPEnable:SetScale(1.5)
	checkboxPEnable:SetPoint("TOPLEFT", 270, -12)
	--power ooc--
	local checkboxPOOC = self:createCheckBox(hFrameOptions, "PShow out of combat", "Show power value when out of combat", self.db.powerFrame.ooc,
	function(val)
		self.db.powerFrame.ooc = val
		if not val and not UnitAffectingCombat("player") then
			SnorkHUDPowerFrame:Hide()
		else
			SnorkHUDPowerFrame:Show()
		end
	end)
	checkboxPOOC:SetPoint("TOPLEFT", 416, -56)
	--power pct--
	local checkboxPPercent = self:createCheckBox(hFrameOptions, "PPercentage", "Show power value as percentage", self.db.powerFrame.pct,
	function(val)
		self.db.powerFrame.pct = val
	end)
	checkboxPPercent:SetPoint("TOPLEFT", 416, -76)
	--power fontsize--
	local sliderPFontSize = self:createSlider(hFrameOptions, "PFont size", "Text size for power value", self.db.powerFrame.size,
	function(val)
		self.db.powerFrame.size = val
		self:initPower()
	end, 8, 60, 2)
	sliderPFontSize:SetPoint("TOPLEFT", 416, -120)
	--power x position--
	local editPX = self:createSlider(hFrameOptions, "PX pos", "Horizontal position for power value", self.db.powerFrame.x,
	function(val)
		self.db.powerFrame.x = val
		self:initPower()
	end, 0, floor(GetScreenWidth()), 2)
	editPX:SetPoint("TOPLEFT", 416, -152)
	--power y position--
	local editPY = self:createSlider(hFrameOptions, "PY pos", "Vertical position for power value", self.db.powerFrame.y,
	function(val)
		self.db.powerFrame.y = val
		self:initPower()
	end, 0, floor(GetScreenHeight()), 2)
	editPY:SetPoint("TOPLEFT", 416, -184)
	--override power color--
	local checkboxPOverride = self:createCheckBox(hFrameOptions, "POverride power color", "Use a specific power color instead of the defaults", self.db.powerFrame.overridecolor,
	function(val)
		self.db.powerFrame.overridecolor = val
		self:initPower()
	end)
	checkboxPOverride:SetPoint("TOPLEFT", 416, -216)
	--power color picker--
	local colorP = self:createColorPicker(hFrameOptions, "Power color", "Color for power value", self.db.powerFrame,
	function(restore)
		local newR, newG, newB, newA;
		if restore then
			newR, newG, newB, newA = unpack(restore);
		else
			newA, newR, newG, newB = OpacitySliderFrame:GetValue(), ColorPickerFrame:GetColorRGB();
		end

		self.db.powerFrame.r = newR
		self.db.powerFrame.g = newG
		self.db.powerFrame.b = newB
		self.db.powerFrame.a = newA
		self:initPower()
	end)
	colorP:SetPoint("TOPLEFT", 432, -236)
	--power outline
	local powerOutline = {"MONOCHROME", "OUTLINE", "THICKOUTLINE"}
	local powerDropOutline = self:createDropDown(hFrameOptions, "PowerOutlineDrop", "Outline of power value", self.db.powerFrame.flags, powerOutline,
	function(key)
		self.db.powerFrame.flags = key.value
		self:initPower()

		UIDropDownMenu_SetSelectedName(PowerOutlineDrop, key.text, key.value)
		UIDropDownMenu_SetText(PowerOutlineDrop, key.value)
		CloseDropDownMenus()
	end)
	powerDropOutline:SetPoint("TOPLEFT", 400, -268)
	--power font--
	local powerFonts = _sml:List("font")
	local powerFontOption = self:createDropDown(hFrameOptions, "PowerFontsDrop", "Font of power value", self.db.powerFrame.font, powerFonts,
	function(key)
		self.db.powerFrame.font = key.value
		self:initPower()

		UIDropDownMenu_SetSelectedName(PowerFontsDrop, key.text, key.value)
		UIDropDownMenu_SetText(PowerFontsDrop, key.value)
		CloseDropDownMenus()
	end)
	powerFontOption:SetPoint("TOPLEFT", 400, -308)

end

--SLASH COMMANDS --
SlashCmdList["SNORKHUD_SLASHCMD"] = function(msg)
	InterfaceOptionsFrame_OpenToCategory("Snork HUD")
end
SLASH_SNORKHUD_SLASHCMD1 = "/snorkhud"
SLASH_SNORKHUD_SLASHCMD2 = "/shud"

_frame:RegisterEvent("PLAYER_LOGIN")
_frame:RegisterEvent("ADDON_LOADED")
_frame:SetScript("OnEvent", _frame.OnEvent)
